@@ -80,8 +80,12 @@ pub enum DecodingError {
     InvalidVersion,
     #[error("invalid schema character {0}")]
     InvalidSchemaCharacter(char),
+    #[error("invalid chunk string {0}")]
+    InvalidChunkString(String),
     #[error("invalid UTF-8 string {0}")]
     InvalidUtf8String(String),
+    #[error("invalid bool {0}")]
+    InvalidBool(String),
 }
 
 /// Atomic parameter in the Airnode ABI
@@ -176,11 +180,11 @@ impl Param {
     /// returns character of the parameter for encoding
     /// - Upper case letters refer to dynamically sized types
     /// - Lower case letters refer to statically sized types
-    /// - String32 is encoded into Bytes32
+    /// - String32 is encoded into Bytes32 in solidity
     pub fn get_char(&self) -> char {
         match &self {
             Self::Address { name: _, value: _ } => 'a',
-            Self::Bool { name: _, value: _ } => 'b',
+            Self::Bool { name: _, value: _ } => 'f',
             Self::Bytes { name: _, value: _ } => 'B',
             Self::Bytes32 { name: _, value: _ } => 'b',
             Self::Date {
@@ -195,7 +199,7 @@ impl Param {
                 sign: _,
             } => 'i',
             Self::String { name: _, value: _ } => 'S',
-            Self::String32 { name: _, value: _ } => 'B',
+            Self::String32 { name: _, value: _ } => 's',
             Self::Uint256 { name: _, value: _ } => 'u',
         }
     }
@@ -274,11 +278,16 @@ impl fmt::Display for Param {
     }
 }
 
+fn default_version() -> u8 {
+    return 1;
+}
+
 /// Airnode ABI object that can be encoded into the vector of U256 and decoded from it
 #[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct ABI {
     /// Id of the ABI version. It is always "1" so far
     #[serde(skip_serializing)]
+    #[serde(default = "default_version")]
     pub version: u8,
     /// Schema string. Each parameter is represented by a char
     pub schema: String,
@@ -319,7 +328,7 @@ impl ABI {
         Self {
             version: 0x31,
             schema: encode_schema(0x31, &params),
-            params: params,
+            params,
         }
     }
 
@@ -399,7 +408,7 @@ impl ABI {
 
         let schema: String = match chunk_to_str(*schema_chunk) {
             Ok(x) => x,
-            Err(e) => return Err(DecodingError::InvalidUtf8String(e.to_string())),
+            Err(e) => return Err(DecodingError::InvalidChunkString(e.to_string())),
         };
         let mut params: Vec<Param> = vec![];
         if schema.len() > 1 {
@@ -433,7 +442,7 @@ impl ABI {
     ) -> Result<Param, DecodingError> {
         let name: String = match chunk_to_str(arr[*offset]) {
             Ok(x) => x,
-            Err(e) => return Err(DecodingError::InvalidUtf8String(e.to_string())),
+            Err(e) => return Err(DecodingError::InvalidChunkString(e.to_string())),
         };
         *offset += 1;
         if ch == 'b' {
@@ -459,6 +468,22 @@ impl ABI {
                 }
             }
             return Ok(Param::Bytes32 { name, value });
+        } else if ch == 'f' {
+            let value = arr[*offset];
+            *offset += 1;
+            match chunk_to_str(value) {
+                Ok(v) => {
+                    if v == "true" {
+                        return Ok(Param::Bool { name, value: true });
+                    } else if v == "false" {
+                        return Ok(Param::Bool { name, value: false });
+                    }
+                    return Err(DecodingError::InvalidBool(v));
+                }
+                Err(e) => {
+                    return Err(DecodingError::InvalidChunkString(e.to_string()));
+                }
+            };
         } else if ch == 'u' {
             let value = arr[*offset];
             *offset += 1;
@@ -471,6 +496,13 @@ impl ABI {
             let (value, sign) = chunk_to_int(arr[*offset]);
             *offset += 1;
             return Ok(Param::Int256 { name, value, sign });
+        } else if ch == 's' {
+            let value = arr[*offset];
+            *offset += 1;
+            match chunk_to_str(value) {
+                Ok(s) => return Ok(Param::String32 { name, value: s }),
+                Err(e) => return Err(DecodingError::InvalidChunkString(e.to_string())),
+            };
         } else if ch == 'B' || ch == 'S' {
             let value_index: usize = arr[*offset].as_usize(); // todo: handle failure
             *offset += 1;
@@ -524,6 +556,17 @@ mod tests {
         assert_eq!(decoded, value);
     }
 
+    #[test]
+    fn it_encodes_decodes_string32() {
+        let param = Param::String32 {
+            name: rand_str(),
+            value: rand_str(),
+        };
+        let value = ABI::only(param);
+        let decoded = ABI::decode(&value.encode().unwrap(), true).unwrap();
+        assert_eq!(decoded, value);
+    }
+
     #[test]
     fn it_encodes_decodes_string() {
         let param = Param::String {
@@ -701,7 +744,7 @@ mod tests {
             hex!("54657374427974657333324e616d650000000000000000000000000000000000").into(),
             hex!("536f6d6520627974657333322076616c75650000000000000000000000000000").into(),
         ];
-        let res = ABI::decode(&data, false).unwrap(); // strict mode "off" is importnant
+        let res = ABI::decode(&data, false).unwrap(); // strict mode "off"
         let expected = ABI::only(Param::String32 {
             name: "TestBytes32Name".to_owned(),
             value: "Some bytes32 value".to_owned(),
@@ -740,7 +783,7 @@ mod tests {
     }
 
     #[test]
-    fn it_encodes_decodes_bool() {
+    fn it_encodes_decodes_true() {
         let param = Param::Bool {
             name: "some bool".to_owned(),
             value: true,
@@ -748,10 +791,25 @@ mod tests {
         let value = ABI::only(param);
         let decoded = ABI::decode(&value.encode().unwrap(), false).unwrap();
         assert_eq!(decoded, value);
+        let decoded = ABI::decode(&value.encode().unwrap(), true).unwrap();
+        assert_eq!(decoded, value);
     }
 
     #[test]
-    fn it_decodes_true() {
+    fn it_encodes_decodes_false() {
+        let param = Param::Bool {
+            name: "some bool".to_owned(),
+            value: false,
+        };
+        let value = ABI::only(param);
+        let decoded = ABI::decode(&value.encode().unwrap(), false).unwrap();
+        assert_eq!(decoded, value);
+        let decoded = ABI::decode(&value.encode().unwrap(), true).unwrap();
+        assert_eq!(decoded, value);
+    }
+
+    #[test]
+    fn it_decodes_true_from_bytes() {
         let data: Vec<U256> = vec![
             hex!("3162000000000000000000000000000000000000000000000000000000000000").into(),
             hex!("54657374426F6F6C000000000000000000000000000000000000000000000000").into(),
@@ -766,7 +824,7 @@ mod tests {
     }
 
     #[test]
-    fn it_decodes_false() {
+    fn it_decodes_false_from_bytes() {
         let data: Vec<U256> = vec![
             hex!("3162000000000000000000000000000000000000000000000000000000000000").into(),
             hex!("54657374426F6F6C000000000000000000000000000000000000000000000000").into(),
